# Copyright (c) 2002 Nuxeo SARL <http://nuxeo.com>
# Copyright (c) 2002 Prfecture du Bas-Rhin, France
# Copyright (c) 2002 CIRB, Belgique
# Authors: Florent Guillaume <mailto:fg@nuxeo.com>
#          Julien Jalon <mailto:jj@nuxeo.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: SkinnerDocument.py,v 1.10 2003/07/14 21:09:03 fermigier Exp $

"""
  Special documents that skins Hierarchies
"""

from zLOG import LOG, INFO

from BaseDocument import BaseDocument, BaseDocument_adder
from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
from Acquisition import aq_base

from Products.CMFCore.CMFCorePermissions import View, ModifyPortalContent

base_factory_type_information = (
    {'id': 'Base Skinner Document',
     'description': 'A document that skins supported containers.',
     'meta_type': 'Base Skinner Document',
     'content_icon': 'document_icon.gif',
     'product': 'NuxCPSDocuments',
     'factory': 'addBaseSkinnerDocument',
     'immediate_view': 'baseskinnerdocument_edit_form',
     'allow_discussion': 0,
     'filter_content_types': 0,
     'actions': ({'id': 'view',
                  'name': '_action_view_',
                  'action': 'baseskinnerdocument_view',
                  'permissions': (View,)},
                 {'id': 'edit',
                  'name': '_action_modify_',
                  'action': 'baseskinnerdocument_edit_form',
                  'permissions': (ModifyPortalContent,)},
                 {'id': 'create',
                  'name': '_action_create_',
                  'action': 'baseskinnerdocument_create_form',
                  'visible': 0,
                  'permissions': ()},
                 {'id': 'isfunctionalobject',
                  'name': 'isfunctionalobject',
                  'action': 'isfunctionalobject',
                  'visible': 0,
                  'permissions': ()},
                 )
     },
    )

# Base for all skinner documents
class BaseSkinnerDocument(BaseDocument):
    """
    Base class for skinners
    """

    meta_type = 'Base Skinner Document'

    security = ClassSecurityInfo()

    isSkinner = 1
    isPrincipiaFolderish = 0

    def manage_afterAdd(self, item, container):
        """Registers item in the container."""
        LOG('SKINNER', INFO, 'Adding "%s" to "%s"' 
            % (getattr(item, 'id', '???'), getattr(container,'id', '???')))
        BaseDocument.manage_afterAdd(self, item, container)
        if item is self:
            LOG('SKINNER', INFO, "That's me!")
            container.skinnerdocument = self.id

    def manage_beforeDelete(self, item, container):
        """Eventually unregisters item from container."""
        LOG('SKINNER', INFO, 'Removing "%s" from "%s"' 
            % (getattr(item, 'id', '???'), getattr(container,'id', '???')))
        BaseDocument.manage_beforeDelete(self, item, container)
        if item is self:
            LOG('SKINNER', INFO, "That's me!")
            if container.skinnerdocument is self.id:
                LOG('SKINNER', INFO, "And I'm the skinner!")
                LOG('SKINNER', INFO, "Looking for an other skinner")
                del container.skinnerdocument
                items = container.objectItems()
                items.reverse()
                for id, document in items:
                    if aq_base(getattr(document, 'isSkinner', 0)):
                        LOG('SKINNER', INFO, "Found %s" % id)
                        container.skinnerdocument = id
                        break

    security.declareProtected(View, 'skin_document')
    def skin_container(self, container, item, doc_infos):
        return "Abstract skinner (%s)" % (self.title_or_id(), )

    security.declareProtected(ModifyPortalContent, 'setSkin')
    def setSkin(self, container):
        if getattr(container, 'skinnerdocument', None) != self.id:
            LOG('SKINNER', INFO, 'Changing skin from %s to %s' % (container.id, self.id))
            container.skinnerdocument = self.id

InitializeClass(BaseSkinnerDocument)


def addBaseSkinnerDocument(dispatcher, id, REQUEST=None, **kw):
    """Add a Base Skinner Document"""
    ob = BaseSkinnerDocument(id, **kw)
    return BaseDocument_adder(dispatcher, id, ob, REQUEST=REQUEST)

# Concrete skinner class. Simply uses 
factory_type_information = (
    {'id': 'Skinner Document',
     'description': 'A document that skins supported containers with a template.',
     'title': '_portal_type_Skinner Document',
     'meta_type': 'Skinner Document',
     'content_icon': 'document_icon.gif',
     'product': 'NuxCPSDocuments',
     'factory': 'addSkinnerDocument',
     'immediate_view': 'skinnerdocument_edit_form',
     'allow_discussion': 0,
     'filter_content_types': 0,
     'actions': ({'id': 'view',
                  'name': '_action_view_',
                  'action': 'skinnerdocument_view',
                  'permissions': (View,)},
                 {'id': 'edit',
                  'name': '_action_modify_',
                  'action': 'skinnerdocument_edit_form',
                  'permissions': (ModifyPortalContent,)},
                 {'id': 'create',
                  'name': '_action_create_',
                  'action': 'skinnerdocument_create_form',
                  'visible': 0,
                  'permissions': ()},
                 {'id': 'isfunctionalobject',
                  'name': 'isfunctionalobject',
                  'action': 'isfunctionalobject',
                  'visible': 0,
                  'permissions': ()},
                 )
     },
    )

# for all skinner documents
class SkinnerDocument(BaseSkinnerDocument):
    """class for documents."""

    meta_type = 'Skinner Document'

    security = ClassSecurityInfo()

    _properties = BaseSkinnerDocument._properties + (
        {'id': 'template', 'type': 'text', 'mode': 'w', 'label': 'Template'},
        )

    template = ''

    security.declareProtected(View, 'skin_document')
    def skin_container(self, container, item, doc_infos):
        # XXX: this should be refactored
        template = self.template
        if template:
            render = getattr(self, template, None)
            if render is not None:
                try:
                    if getattr(aq_base(render), 'isDocTemp', 0):
                        return render(self, self.REQUEST, container=container, 
                                      item=item, doc_infos=doc_infos)
                    else:
                        return render(container=container, item=item, 
                                      doc_infos=doc_infos)
                except:
                    # XXX show the error stack ?
                    raise
                    return "Error in template %s" % (template, )
            else:
                return "Template %s not found" % (template, )
        else:
            return "No template provided"

InitializeClass(SkinnerDocument)


def addSkinnerDocument(dispatcher, id, REQUEST=None, **kw):
    """Add a Skinner Document."""
    ob = SkinnerDocument(id, **kw)
    return BaseDocument_adder(dispatcher, id, ob, REQUEST=REQUEST)

