# Copyright (c) 2002 Nuxeo SARL <http://nuxeo.com>
# Copyright (c) 2002 Prfecture du Bas-Rhin, France
# Copyright (c) 2002 CIRB, Belgique
# Authors: Florent Guillaume <mailto:fg@nuxeo.com>
#          Julien Jalon <mailto:jj@nuxeo.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: __init__.py,v 1.14 2003/08/09 07:13:07 fermigier Exp $

import sys

from Products.CMFCore import utils
from Products.CMFCore.DirectoryView import registerDirectory
from Products.CMFCore.CMFCorePermissions import AddPortalContent

import BaseDocument
import TextDocument
import NewsDocument
import FAQDocument
import FileDocument
import ContactDocument
import URLDocument
import MailingList
# Newsletter currently doesn't work on Windows
try:
    import Newsletter
except ImportError:
    Newsletter = None
import PatchDiscussionItem
import SkinnerDocument

contentClasses = (BaseDocument.BaseDocument,
                  TextDocument.TextDocument,
                  NewsDocument.NewsDocument,
                  FAQDocument.FAQDocument,
                  FileDocument.FileDocument,
                  ContactDocument.ContactDocument,
                  URLDocument.URLDocument,
                  MailingList.MailingList,
                  SkinnerDocument.BaseSkinnerDocument,
                  SkinnerDocument.SkinnerDocument,
                  )
if Newsletter:
    contentClasses += (Newsletter.Newsletter,)

contentConstructors = (BaseDocument.addBaseDocument,
                       TextDocument.addTextDocument,
                       NewsDocument.addNewsDocument,
                       FAQDocument.addFAQDocument,
                       FileDocument.addFileDocument,
                       ContactDocument.addContactDocument,
                       URLDocument.addURLDocument,
                       MailingList.addMailingList,
                       SkinnerDocument.addBaseSkinnerDocument,
                       SkinnerDocument.addSkinnerDocument,
                       )
if Newsletter:
    contentConstructors += (Newsletter.addNewsletter,)

fti = (BaseDocument.factory_type_information +
       TextDocument.factory_type_information +
       NewsDocument.factory_type_information +
       FAQDocument.factory_type_information +
       FileDocument.factory_type_information +
       ContactDocument.factory_type_information +
       URLDocument.factory_type_information +
       MailingList.factory_type_information +
       SkinnerDocument.base_factory_type_information +
       SkinnerDocument.factory_type_information +
       ()
       )
if Newsletter:
    fti += Newsletter.factory_type_information

bases = (BaseDocument.BaseDocument,) # base zclasses

this_module = sys.modules[__name__]
z_bases = utils.initializeBasesPhase1(bases, this_module)

registerDirectory('skins', globals())

def initialize(registrar):
    utils.initializeBasesPhase2(z_bases, registrar)
    utils.ContentInit(
        'NuxCPSDocuments',
        content_types = contentClasses,
        permission = AddPortalContent,
        extra_constructors = contentConstructors,
        fti = fti,
        ).initialize(registrar)
