# Copyright (c) 2002 Nuxeo SARL <http://nuxeo.com>
# Copyright (c) 2002 CIRB, Belgique
# Author: Julien Jalon <mailto:jj@nuxeo.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: CPSMembersPrivateArea.py,v 1.8 2003/06/12 09:17:33 fermigier Exp $

"""
  CPSMembersPrivateArea

  Patches the CMF Default Membership Tool to make the member's private
  area a Workgroup
"""

from zLOG import LOG, INFO

from Products.CMFDefault.MembershipTool import MembershipTool
from Products.NuxWorkgroup import Workgroup
from Products.CMFCore.utils import getToolByName


WorkgroupManager = Workgroup.WorkgroupManager
WorkgroupMember = Workgroup.WorkgroupMember
WorkgroupVisitor = Workgroup.WorkgroupVisitor
WorkgroupPortalType = Workgroup.factory_type_information[0]['id']

LOG("NuxCPSMembersPrivateArea", INFO, "Patching CMF Default Membership Tool: PortalFolder -> %s" % (WorkgroupPortalType, ))

def createMemberarea(self, member_id):
    """
    create a member area
    """
    parent = self.aq_inner.aq_parent
    members =  getattr(parent, 'Members', None)

    if members is not None and not hasattr(members, member_id):
        privatearea_title_process = getattr(self, 'privatearea_title_process', None)

        if privatearea_title_process is not None:
            f_title = privatearea_title_process(member_id = member_id)
        else:
            f_title = "_private_area_"

        LOG("NuxCPSMembersPrivateArea", INFO, "Creating private area for %s in %s" % (member_id, self.id))
        # do exactly what
        # members.invokeFactory(WorkgroupPortalType, member_id)
        # does but without the permissions validation
        # XXX should be a better way to do that :-/
        portal_types = getToolByName(members, 'portal_types')
        type_information = portal_types.getTypeInfo(WorkgroupPortalType)
        product = members.manage_addProduct[type_information.product]
        factory_method = getattr(product, type_information.factory)
        factory_method(member_id, title = f_title)

        private_area = getattr(members, member_id)

        # Fix private_area portal type
        private_area._setPortalTypeName(type_information.getId())

        # Grant ownership to Member
        acl_users = self._MembershipTool__getPUS()
        user = acl_users.getUser(member_id).__of__(acl_users)

        mtool = getToolByName(self, 'portal_membership')
        if not mtool.isAnonymousUser():
            # remove current user from this private area local roles
            current_user = mtool.getAuthenticatedMember().getUserName()
            private_area.manage_delLocalRoles(userids=[current_user])

        private_area.changeOwnership(user)
        private_area.manage_setLocalRoles(member_id, ['Owner', WorkgroupManager, WorkgroupMember, WorkgroupVisitor])
        private_area._computedtitle = 1

        # Register the private area in the portal_workflow
        portal_workflow = getToolByName(private_area, 'portal_workflow')
        if portal_workflow is not None:
            portal_workflow.notifyCreated(private_area)

        # Index the private_area in the catalog
        private_area.reindexObject()

MembershipTool.createMemberarea = createMemberarea
