# Copyright (c) 2001, 2002 Nuxeo SARL <http://nuxeo.com>
# Copyright (c) 2001, 2002 Florent Guillaume <mailto:fg@nuxeo.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: PortalContentFolder.py,v 1.6 2002/08/02 17:44:34 guillaum Exp $

"""PortalContentFolder

This class (for CMF) is a mix of both PortalContent and PortalFolder,
so that you can have a folder-like object that gets cataloged and is
subject to the workflow.
"""

from AccessControl import ClassSecurityInfo
from Globals import InitializeClass

from Products.CMFCore.CMFCorePermissions import View, ModifyPortalContent
from Products.CMFCore.CMFCatalogAware import CMFCatalogAware
from Products.CMFCore.PortalFolder import PortalFolder
from Products.CMFCore.PortalContent import PortalContent
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl


def addPortalContentFolder(self, id, title='', REQUEST=None):
    """
    Adds a new PortalContentFolder.
    """
    ob = PortalContentFolder(id, title)
    self._setObject(id, ob)
    if REQUEST is not None:
        REQUEST.RESPONSE.redirect(self.absolute_url()+'/manage_main')


# addPortalContentFolderForm = DTMLFile('dtml/addPortalContentFolderForm', globals())


factory_type_information = (
    { 'id'             : 'PortalContentFolder',
      'meta_type'      : 'PortalContentFolder',
      'description'    :
      """A PortalContentFolder has content and behaves like a folder.""",
      'icon'           : '',
      'product'        : 'PortalContentFolder',
      'factory'        : 'addPortalContentFolder',
      'filter_content_types' : 0,
      'immediate_view' : 'folder_contents',
      'actions'        :
      ( { 'name'          : 'View',
          'action'        : 'folder_contents',
          'permissions'   : (View,),
          'category'      : 'object',
          },
        { 'name'          : 'Edit',
          'action'        : 'folder_edit_form',
          'permissions'   : (ModifyPortalContent,),
          },
        { 'name'          : 'Metadata',
          'action'        : 'metadata_edit_form',
          'permissions'   : (ModifyPortalContent,),
          },
        ),
      },
    )


class PortalContentFolder(CMFCatalogAware, PortalFolder, PortalContent,
                          DefaultDublinCoreImpl):
    """
    Implements a PortalContentFolder, a PortalFolder with contentish
    methods so that it can belong to a workflow and be indexed.
    """

    meta_type = 'PortalContentFolder'
    portal_type = 'PortalContentFolder'

    manage_options = (PortalFolder.manage_options[:3] +
                      ({'label'  : 'Dublin Core',
                        'action' : 'manage_metadata',
                        },
                       ) +
                      PortalFolder.manage_options[3:]
                      )

    security = ClassSecurityInfo()

    # derived classes should call this
    def __init__(self, id, title='', **kw):
        self.id = id
        DefaultDublinCoreImpl.__init__(self)
        self.title = title

    #
    # Misc
    #

    security.declarePublic('getIconRelative')
    def getIconRelative(self):
        """
        Gets the Icon path, relative to the portal.
        This is needed to catalog correctly in the presence of VHM.
        """
        return self.getIcon(1)


InitializeClass(PortalContentFolder)
