#!/usr/bin/env python
##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
# 
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
# 
##############################################################################
"""
$Id: clock.py,v 1.1 2003/03/12 11:02:29 smigniot Exp $

External clock tick component of Scheduler product.  Requires Python 2.2.

"""
__version__ = "$Revision: 1.1 $"[11:-2]

import os
import sys
import time
from xmlrpclib import Server
from BasicAuthTransport import BasicAuthTransport
import getopt
import traceback

class Clock:

    def __init__( self
                , scheduler_url
                , period
                , userid
                , password
                , pidfile
                , logfile=sys.stdout
                , errfile=sys.stderr
                , verbosity=0
                ):

        self._scheduler_url = scheduler_url
        self._period = period
        self._userid = userid
        self._password = password
        self._logfile = logfile
        self._errfile = errfile
        self._verbosity = verbosity
        self._pidfile = pidfile

    def run(self):
        """ Loop forever, popping the scheduler and then sleeping.
        """

        try:
            pidfile = open(self._pidfile,"w+")
            pidfile.write(str(os.getpid()))
            pidfile.close()
        except:
            traceback.print_exc(file=self._errfile)
            self._errfile.write( '\n' )
            self._logfile.write("Unable to write pid = [%s] pidfile '%s'. "
                    % (os.getpid(), self._pidfile))

        scheduler = self._makeScheduler()
        while 1:
            time.sleep(self._period)
            try:
                scheduler.notify()
            except:
                traceback.print_exc(file=self._errfile)
                self._errfile.write( '\n' )
                self._logfile.write('Clock failure at %s\n'
                                   % time.ctime(time.time()))
            else:
                if self._verbosity:
                    self._logfile.write('Clock succeded at %s\n'
                                       % time.ctime(time.time()))

    def _makeScheduler(self):
        """ Create the XML-RPC connection to the scheduler.
        """
        #   XXX:  Hardwired credentials
        transport = BasicAuthTransport(self._userid, self._password)
        return Server(self._scheduler_url, transport=transport)


def usage():
    USAGE = """clock [options]

Options:

  -?, -h, --help        Print this usage message.

  -q, --quiet           Don't blather about success (default).

  -v, --verbose         Blather about success.

  -n, --nethost         Supply the nethost part of the scheduler URL
                        (defaults to 'http://localhost:8080').

  -s, --scheduler_path  Supply the path part of the scheduler URL
                        (defaults to '/workbench/portal_scheduler').

  -a, --auth            Supply the authentication credentials as a
                        'userid:password' token.

  -p, --period          Supply the period interval, in seconds, at which
                        the clock should notify the schedulre (default 20).

  -P, --pidfile         Supply the pid file
                        (defaults to './var/clock.pid')

  -U, --url             Supply the full URL
                        (defaults to nethost/scheduler_path)
"""
    print USAGE
    sys.exit(2)

def main():

    url = 'http://localhost:8080/cps/portal_scheduler'
    userid = 'admin'
    password = '123'
    period = 30
    verbosity = 0
    pidfile = './var/clock.pid'

    try:
        opts, args = getopt.getopt( sys.argv[1:]
                                , '?hqva:p:P:U:'
                                , [ 'help'
                                  , 'quiet'
                                  , 'verbose'
                                  , 'auth='
                                  , 'period='
                                  , 'pidfile='
                                  , 'url='
                                  ]
                                )
    except getopt.GetoptError:
        usage()

    if args:
        usage()

    for k, v in opts:

        if k == '-h' or k == '-?' or k == '--help':
            usage()

        if k == '-q' or k == '--quiet':
            verbosity = 0

        if k == '-v' or k == '--verbose':
            verbosity += 1

        if k == '-a' or k == '--auth':
            userid, password = v.split( ':' )

        if k == '-p' or k == '--period':
            period = int( v )

        if k == '-P' or k == '--pidfile':
            pidfile = v

        if k == '-U' or k == '--url':
            url = v

    Clock( scheduler_url=url
         , period=period
         , userid=userid
         , password=password
         , pidfile=pidfile
         , logfile=sys.stdout
         , errfile=sys.stderr
         , verbosity=verbosity
         ).run()

if __name__ == '__main__':
    main()
